/***************************************************************************
 *   Copyright (C) 2004, 2005 Thomas Nagy                                  *
 *   tnagy2^8@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include <qpainter.h>
#include <qcolor.h>
#include <kdebug.h>
#include "DCanvasItem.h"
#include "DCanvasLink.h"
#include "settings.h"

DCanvasLink::DCanvasLink(QCanvas* canvas, DCanvasItem* orig) : QCanvasPolygon(canvas)
{
	m_orig = orig;
	m_dest = NULL;

	move(0,0);
	m_pa.putPoints(0, 4, 0,0,  0,0,  0,0,  0,0 );
	updateColor();
}

DCanvasLink::~DCanvasLink()
{
	hide();
}

void DCanvasLink::setDest(DCanvasItem* dest)
{
	m_dest = dest;
}

void DCanvasLink::updateColor()
{
	QBrush brush;
	if ( m_orig->control()->m_reverseVideo )
		brush.setColor( QColor(Qt::white) );
	else
		brush.setColor( m_orig->control()->m_col_link );
	brush.setStyle( QBrush::SolidPattern );
	setBrush(brush);
}

void DCanvasLink::updatePos()
{
	if (!m_dest)
	{
		setPosHide();
		return;
	}

	if ( m_orig->control()->m_linkstyle == Settings::EnumLinkStyle::spline )
	{
		setPosWeb();
		// setPosCurves();
	}
	else if (false)
	{
		setPosWeb();
	}
	else
	{
		setPosLines();
	}
}

void DCanvasLink::setPosHide()
{
	for (int i=0; i<4; i++) m_pa[i] = QPoint(0, 0);
	setPoints(m_pa);
	move(0, 0);
}

int DCanvasLink::IdOrig()
{
	return m_orig->Id();
}

int DCanvasLink::IdDest()
{
	return m_dest->Id();
}

void DCanvasLink::setPosLines()
{
	//kdWarning()<<"DCanvasLink::setPosLines"<<endl;

	// join the middles of the two rectangles
	double x1 = m_orig->x() + m_orig->width()/2;
	double y1 = m_orig->y() + m_orig->height()/2;
	double x2 = m_dest->x() + m_dest->width()/2;
	double y2 = m_dest->y() + m_dest->height()/2;

	double X1 = x2 - x1;
	double Y1 = y2 - y1;

	if (QABS(X1) < 10 && QABS(Y1) < 10)
	{
		setPosHide();
		return;
	}

	double dist = 1;
	double coeff = Y1*Y1/(Y1*Y1+X1*X1);

	double deltax = dist * sqrt( coeff );
	double deltay = dist * sqrt( 1 - coeff );

	if (X1 != 0) if (Y1/X1 < 0) deltay *= -1;

	int s = m_orig->control()->m_showDirectionSize;

	m_pa[0] = QPoint((int) (x1 + deltax), (int) (y1 - deltay));
	m_pa[1] = QPoint((int) (x2 + s*deltax), (int) (y2 - s*deltay));
	m_pa[2] = QPoint((int) (x2 - s*deltax), (int) (y2 + s*deltay));
	m_pa[3] = QPoint((int) (x1 - deltax), (int) (y1 + deltay));

	setPoints(m_pa);
}

void DCanvasLink::setPosWeb()
{
	int s = m_orig->control()->m_showDirectionSize;

	int diffx=0, diffy=0;
	QPointArray pa(4);
	QPointArray pb(4);
	QPointArray pc(8);

	if ( m_dest->x() > 10 + m_orig->x() + m_orig->width() )
	{
		// parent is on the left
		pa[0]=QPoint( (int)(m_orig->x() /*+ m_orig->width()*/), (int)(m_orig->y() + m_orig->height()/2 +1));
		pa[1]=QPoint( (int)(m_dest->x()                  ), (int)(m_orig->y() + m_orig->height()/2 +s));
		pa[2]=QPoint( (int)(m_dest->x()                  ), (int)(m_dest->y() + m_dest->height()/2 +s));
		pa[3]=QPoint( (int)(m_dest->x()                  ), (int)(m_dest->y() + m_dest->height()/2 +s));

		pb[3]=QPoint( (int)(m_orig->x() /*+ m_orig->width()*/), (int)(m_orig->y() + m_orig->height()/2 -1));
		pb[2]=QPoint( (int)(m_dest->x()                  ), (int)(m_orig->y() + m_orig->height()/2 -s));
		pb[1]=QPoint( (int)(m_dest->x()                  ), (int)(m_dest->y() + m_dest->height()/2 -s));
		pb[0]=QPoint( (int)(m_dest->x()                  ), (int)(m_dest->y() + m_dest->height()/2 -s));

		QPointArray bez1 = pa.cubicBezier();
		QPointArray bez2 = pb.cubicBezier();

		QPointArray poly(bez1.count()+bez2.count());
		int n=0;
		for (int i=0; i<bez1.count(); i++)
		{
			poly[n++] = bez1.point(i);
		}
		for (int i=0; i<bez2.count(); i++)
		{
			poly[n++] = bez2.point(i);
		}
		setPoints( poly );
	}
	else if ( m_dest->x() + m_dest->width() + 10 < m_orig->x() )
	{
		// parent is on the right
		pa[0]=QPoint( (int)m_orig->x() + m_orig->width(), (int)(m_orig->y() + m_orig->height()/2 +1));
		pa[1]=QPoint( (int)m_dest->x() + m_dest->width(), (int)(m_orig->y() + m_orig->height()/2 +s));
		// idea :
		// pa[1]=QPoint( (int)m_dest->x() + m_dest->width(), (int)(m_dest->y() + m_dest->height()/2 +s));

		pa[2]=QPoint( (int)m_dest->x() + m_dest->width(), (int)(m_dest->y() + m_dest->height()/2 +s));
		pa[3]=QPoint( (int)m_dest->x() + m_dest->width(), (int)(m_dest->y() + m_dest->height()/2 +s));

		pb[3]=QPoint( (int)m_orig->x() + m_orig->width(), (int)(m_orig->y() + m_orig->height()/2 -1));
		pb[2]=QPoint( (int)m_dest->x() + m_dest->width(), (int)(m_orig->y() + m_orig->height()/2 -s));
		pb[1]=QPoint( (int)m_dest->x() + m_dest->width(), (int)(m_dest->y() + m_dest->height()/2 -s));
		pb[0]=QPoint( (int)m_dest->x() + m_dest->width(), (int)(m_dest->y() + m_dest->height()/2 -s));

		QPointArray bez1 = pa.cubicBezier();
		QPointArray bez2 = pb.cubicBezier();

		QPointArray poly(bez1.count()+bez2.count());
		int n=0;
		for (int i=0; i<bez1.count(); i++)
		{
			poly[n++] = bez1.point(i);
		}
		for (int i=0; i<bez2.count(); i++)
		{
			poly[n++] = bez2.point(i);
		}
		setPoints( poly );
	}
	else if ( m_dest->y() + m_dest->height() + 10 < m_orig->y() )
	{
		// parent is below
		pa[0]=QPoint( (int)m_orig->x() + m_orig->width()/2+1, (int)(m_orig->y()+m_orig->height()));
		pa[1]=QPoint( (int)m_orig->x() + m_orig->width()/2+s, (int)(m_dest->y()+m_dest->height()));
		pa[2]=QPoint( (int)m_dest->x() + m_dest->width()/2+s, (int)(m_dest->y()+m_dest->height()));
		pa[3]=QPoint( (int)m_dest->x() + m_dest->width()/2+s, (int)(m_dest->y()+m_dest->height()));

		pb[3]=QPoint( (int)m_orig->x() + m_orig->width()/2-1, (int)(m_orig->y()+m_orig->height()));
		pb[2]=QPoint( (int)m_orig->x() + m_orig->width()/2-s, (int)(m_dest->y()+m_dest->height()));
		pb[1]=QPoint( (int)m_dest->x() + m_dest->width()/2-s, (int)(m_dest->y()+m_dest->height()));
		pb[0]=QPoint( (int)m_dest->x() + m_dest->width()/2-s, (int)(m_dest->y()+m_dest->height()));

		QPointArray bez1 = pa.cubicBezier();
		QPointArray bez2 = pb.cubicBezier();

		QPointArray poly(bez1.count()+bez2.count());
		int n=0;
		for (int i=0; i<bez1.count(); i++)
		{
			poly[n++] = bez1.point(i);
		}
		for (int i=0; i<bez2.count(); i++)
		{
			poly[n++] = bez2.point(i);
		}
		setPoints( poly );
	}
	else if ( m_dest->y() > m_orig->y() + m_orig->height() + 10 )
	{
		// parent is above
		pa[0]=QPoint( (int)m_orig->x() + m_orig->width()/2+1, (int)(m_orig->y()));
		pa[1]=QPoint( (int)m_orig->x() + m_orig->width()/2+s, (int)(m_dest->y()));
		pa[2]=QPoint( (int)m_dest->x() + m_dest->width()/2+s, (int)(m_dest->y()));
		pa[3]=QPoint( (int)m_dest->x() + m_dest->width()/2+s, (int)(m_dest->y()));

		pb[3]=QPoint( (int)m_orig->x() + m_orig->width()/2-1, (int)(m_orig->y()));
		pb[2]=QPoint( (int)m_orig->x() + m_orig->width()/2-s, (int)(m_dest->y()));
		pb[1]=QPoint( (int)m_dest->x() + m_dest->width()/2-s, (int)(m_dest->y()));
		pb[0]=QPoint( (int)m_dest->x() + m_dest->width()/2-s, (int)(m_dest->y()));

		QPointArray bez1 = pa.cubicBezier();
		QPointArray bez2 = pb.cubicBezier();

		QPointArray poly(bez1.count()+bez2.count());
		int n=0;
		for (int i=0; i<bez1.count(); i++)
		{
			poly[n++] = bez1.point(i);
		}
		for (int i=0; i<bez2.count(); i++)
		{
			poly[n++] = bez2.point(i);
		}
		setPoints( poly );
	}
	else
	{
		// when we dont know how to link, use straight lines
		setPosLines();
	}
}

// TODO : this won't work
void DCanvasLink::setPosCurves()
{
	// join the middles of the two rectangles
	double x1 = m_orig->x() + m_orig->width()/2;
	double y1 = m_orig->y() + m_orig->height()/2;
	double x2 = m_dest->x() + m_dest->width()/2;
	double y2 = m_dest->y() + m_dest->height()/2;

	QPoint pa;
	QPoint pb;

	double dec=0;
	bool above = false;
	// above or below
	if (y2 < y1) { y1 = m_orig->y(); dec = y1 + 3; }
	else { above = true; y1 = m_orig->y() + m_orig->height(); dec = y1 - 3; }
	// left or right

	if (x1 < x2)
	{
		pa = QPoint((int) m_orig->x() + m_orig->width(), (int) y1);
		pb = QPoint((int) m_orig->x() + m_orig->width(), (int) dec);
	}
	else
	{
		pa = QPoint((int) m_orig->x(), (int) y1);
		pb = QPoint((int) m_orig->x(), (int) dec);
	}

	double X1 = x2 - x1; double Y1 = y2 - y1;
	if (QABS(X1) < 10 && QABS(Y1) < 10) { setPosHide(); return; }


	double coeff = (Y1*Y1)/(Y1*Y1+X1*X1);
	double deltax = 1 * sqrt( coeff );
	double deltay = 1 * sqrt( 1 - coeff );
	if (X1 != 0) if (Y1/X1 < 0) deltay *= -1;

	QPoint ppa = QPoint((int) (x2 + 4*deltax), (int) (y2 - 4*deltay));
	QPoint ppb = QPoint((int) (x2 - 4*deltax), (int) (y2 + 4*deltay));

	kdWarning()<<"setting the points"<<endl;

	QPointArray polya;
	int wwa = QABS(pa.x()-ppa.x());
	int hha = QABS(pa.y()-ppa.y());
	QPointArray polyb;
	int wwb = QABS(pb.x()-ppb.x());
	int hhb = QABS(pb.y()-ppb.y());

	if (x1 < x2)
	{
		if (pa.y()<y2) 
		{
			polya.makeArc( pa.x()-wwa, pa.y(), 2*wwa, 2*hha, 0, 90*16);
			polyb.makeArc( pb.x()-wwb, pb.y(), 2*wwb, 2*hhb, 0, 90*16);
		}
		else 
		{
			polya.makeArc( pa.x()-wwa, pa.y()-2*hha, 2*wwa, 2*hha, 0, -90*16);
			polyb.makeArc( pb.x()-wwb, pb.y()-2*hhb, 2*wwb, 2*hhb, 0, -90*16);
		}
	}
	else
	{
		if (pa.y()<y2) 
		{
			polya.makeArc( pa.x()-wwa, pa.y(), 2*wwa, 2*hha, 90*16, 90*16);
			polyb.makeArc( pb.x()-wwb, pb.y(), 2*wwb, 2*hhb, 90*16, 90*16);
		}
		else 
		{
			polya.makeArc( pa.x()-wwa, pa.y()-2*hha, 2*wwa, 2*hha, -90*16, -90*16);
			polyb.makeArc( pb.x()-wwb, pb.y()-2*hhb, 2*wwb, 2*hhb, -90*16, -90*16);
		}
	}

	if (polya.count() > 1 && polyb.count()> 1)
	{

		kdWarning()<<polya.count()<<" "<<polyb.count()<<endl;

		kdWarning()<<polya.point(polya.count()-1)<<" "<<polya.point(0)<<endl;

		QPointArray poly(polya.count()+polyb.count()-2);

		int n=0;
		for (int i=0; i<polya.count()-1;i++)
		{
			poly[n] = polya.point(i);
			n++;
			if (n >= poly.count())
				break;
		}
		for (int i=0; i<polyb.count()-1;i++)
		{
			poly[n] = polyb.point(polyb.count()-i);
			n++;

			if (n >= poly.count())
				break;
		}

		setPoints(poly);
	}

	//setPoints(polyb);
}

