/***************************************************************************
 *   Copyright (C) 2004, 2005 Thomas Nagy                                  *
 *   tnagy2^8@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kiconloader.h>
#include <klocale.h>

#include "settings.h"
#include "DCanvasView.h"
#include "DCanvasPopup.h"

DCanvasPopup::DCanvasPopup(QWidget *parent) : KPopupMenu(parent)
{
    m_canvas = (DCanvasView*) parent;

    // new object on the canvas / canvas manipulation
    insertItem(i18n("New item"), e_item);

    // edit actions
    if ( m_canvas->itemsAreSelected() )
    {
	insertSeparator();
	insertItem(i18n("Delete"), e_delete);

	insertSeparator();
    }

    // disconnect actions
    KPopupMenu *disc_pop = new KPopupMenu(this, i18n("Disconnect"));
    
    // disc_pop ~ disconnect popup :)

    disc_pop->insertItem(i18n("Parent"), e_discon_parent);
    disc_pop->insertItem(i18n("Object(s)"), e_discon_obj);
    disc_pop->insertItem(i18n("Subtree"), e_discon_subtree);
    if (m_canvas->selectedRef())
	disc_pop->insertItem(i18n("Reference"), e_discon_link);
    else if (m_canvas->selectedLink())
	disc_pop->insertItem(i18n("Link"), e_discon_link);

    insertItem( i18n("Disconnect"), disc_pop);

    if (disc_pop)
	connect(disc_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));

    setItemEnabled(e_discon_subtree, m_canvas->oneItemIsSelected());
    setItemEnabled(e_discon_parent,  m_canvas->oneItemIsSelected());
    setItemEnabled(e_discon_obj, m_canvas->itemsAreSelected());

    disc_pop->setEnabled( m_canvas->itemsAreSelected() || m_canvas->selectedRef() || m_canvas->selectedLink() );

    insertSeparator();

    // tydy actions
    
    KPopupMenu *tidy_pop = new KPopupMenu(this, i18n("Tidy"));

    tidy_pop->insertItem(i18n("Quick reorganization"), e_tidy_clean);
    tidy_pop->insertItem(i18n("Smart reorganization (tm)"), e_tidy_all);
    tidy_pop->insertItem(i18n("Unclutter the mindmap"), e_tidy_unclutter);


    insertItem( i18n("Tidy"), tidy_pop);
    connect(tidy_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));
    
    tidy_pop->setEnabled(true);
    //tidy_pop->setItemEnabled(e_tidy_unclutter, false);
    insertSeparator();

    //insertItem(i18n("Colors"), e_color);
    //setItemEnabled(e_color, m_canvas->itemsAreSelected() );

    KPopupMenu *col_pop = new KPopupMenu(this, i18n("Colors"));

    col_pop->insertItem(i18n("Default color"), Settings::EnumColorMode::default_);
    col_pop->insertItem(i18n("Important"), Settings::EnumColorMode::important_);
    col_pop->insertItem(i18n("Opposition"), Settings::EnumColorMode::opposition_);
    col_pop->insertItem(i18n("Enumeration"), Settings::EnumColorMode::enumeration_);
    col_pop->insertItem(i18n("Todo"), Settings::EnumColorMode::todo_);
    col_pop->insertItem(i18n("Done"), Settings::EnumColorMode::done_);
    col_pop->insertItem(i18n("Clear"), Settings::EnumColorMode::clear_);
    col_pop->insertSeparator();
    col_pop->insertItem(i18n("Custom..."), Settings::EnumColorMode::custom_);

    if (m_canvas->oneItemIsSelected())
	col_pop->setItemChecked( m_canvas->selectedDataItem()->colorScheme(), true);

    connect(col_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));

    insertItem( i18n("Colors"), col_pop);		
    col_pop->setEnabled( m_canvas->itemsAreSelected() );
    //setItemEnabled(col_pop, m_canvas->itemsAreSelected() );

    insertItem(i18n("Font properties"), e_font);
    setItemEnabled(e_font, m_canvas->itemsAreSelected() );

}

