/***************************************************************************
 *   Copyright (C) 2005 Thomas Nagy                                        *
 *   tnagy2^8@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qfileinfo.h>
#include <qframe.h>

#include <kparts/part.h>
#include <kate/document.h>
#include <kate/view.h>
#include <klocale.h>
#include <klibloader.h>
#include <kurl.h>

#include "KDIKonsole.h"

KDIKonsole::KDIKonsole(QWidget *parent, const char *name) : 
    QVBox(parent, name)
{
    m_haskonsole = false;
    respawn();
}

KDIKonsole::~KDIKonsole()
{
}

void KDIKonsole::respawn()
{
    KLibFactory *factory = KLibLoader::self()->factory("libkonsolepart");

    if (!factory) return;
    m_part = (KParts::ReadOnlyPart *) factory->create(this);

    if (!m_part) return;

    if (m_part->widget()->inherits("QFrame"))
	((QFrame*)m_part->widget())->setFrameStyle(QFrame::Panel|QFrame::Sunken);

    m_haskonsole=true;
    connect( m_part, SIGNAL(destroyed()), this, SLOT(slotDestroyed()) );

    m_part->widget()->show();
    show();
}

void KDIKonsole::setDirectory(const QString &dirname)
{
    if (m_haskonsole)
    {
	KURL url(dirname);
	if (m_part->url() != url)
	    m_part->openURL(url);
    }
}

void KDIKonsole::showEvent(QShowEvent *ev)
{
    QWidget::showEvent(ev);
    activate();
}

void KDIKonsole::activate()
{
    if (m_haskonsole)
    {
	m_part->widget()->show();
	setFocusProxy(m_part->widget());
	m_part->widget()->setFocus();
    }
}

void KDIKonsole::slotDestroyed()
{
    m_haskonsole=false;
    respawn();
}

#include "KDIKonsole.moc"
