/***************************************************************************
 *   Copyright (C) 2004, 2005 Thomas Nagy                                  *
 *   tnagy2^8@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kiconloader.h>
#include <klocale.h>

#include "settings.h"
#include "DTreeListItem.h"
#include "DTreeListView.h"
#include "DTreeListPopup.h"


// TODO the current implementation lets only select one item at a time

DTreeListPopup::DTreeListPopup(QWidget *parent) : KPopupMenu(parent)
{
    m_lstview = (DTreeListView*) parent;

    // new object
    insertItem(i18n("New item"), e_item);

    // edit actions
    if ( m_lstview->selectedItem() )
    {
	insertSeparator();
	insertItem(i18n("Delete"), e_delete);

	insertSeparator();
    }

    // disconnect actions

    KPopupMenu *disc_pop = new KPopupMenu(this, i18n("Disconnect"));
    // disc_pop ~ disconnect popup :)

    disc_pop->insertItem(i18n("Object(s)"), e_discon_obj);
    disc_pop->insertItem(i18n("Subtree"), e_discon_subtree);

    insertItem( i18n("Disconnect"), disc_pop);

    if (disc_pop)
	connect(disc_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));

    setItemEnabled(e_discon_subtree, m_lstview->selectedItem() );

    disc_pop->setEnabled( m_lstview->selectedItem() );

    insertSeparator();

    // properties actions

    KPopupMenu *col_pop = new KPopupMenu(this, i18n("Colors"));

    col_pop->insertItem(i18n("Default color"), Settings::EnumColorMode::default_);
    col_pop->insertItem(i18n("Important"), Settings::EnumColorMode::important_);
    col_pop->insertItem(i18n("Opposition"), Settings::EnumColorMode::opposition_);
    col_pop->insertItem(i18n("Enumeration"), Settings::EnumColorMode::enumeration_);
    col_pop->insertItem(i18n("Todo"), Settings::EnumColorMode::todo_);
    col_pop->insertItem(i18n("Done"), Settings::EnumColorMode::done_);
    col_pop->insertItem(i18n("Clear"), Settings::EnumColorMode::clear_);
    col_pop->insertSeparator();
    col_pop->insertItem(i18n("Custom..."), Settings::EnumColorMode::custom_);

    if (m_lstview->selectedItem())
    {
	DTreeListItem *it = (DTreeListItem*) m_lstview->selectedItem();
        col_pop->setItemChecked( m_lstview->dataItem(it->Id())->colorScheme(), true);
    }

    connect(col_pop, SIGNAL(activated(int)), this, SIGNAL(activated(int)));

    insertItem( i18n("Colors"), col_pop);		
    col_pop->setEnabled( m_lstview->selectedItem() );


    insertItem(i18n("Properties..."), e_properties);
    setItemEnabled(e_properties, m_lstview->selectedItem() );
}

